-------------------------------------------------------------------------------
-- Title      : controlUnit
-- Project    :
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    :
-- Created    : 2018-04-11
-- Last update: 2019-02-13
-- Platform   :
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Control unit of a sequential FIR filter.
-------------------------------------------------------------------------------
-- Copyright (c) 2018
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2018-04-11  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity controlUnit is

  port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSampleValid    : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_processingDoneIRRA  : in  std_logic;
    I_processingDoneIRRB  : in  std_logic;
    I_processingDoneBaseline : in  std_logic;
    I_processingDoneLissage : in  std_logic;
    O_loadShift           : out std_logic;  -- filtered sample
    O_initAddress         : out std_logic;  -- Control signal to initialize register read address
    O_incrAddress         : out std_logic;  -- Control signal to increment register read address
    O_initSum             : out std_logic;  -- Control signal to initialize the MAC register
    O_loadSum             : out std_logic;  -- Control signal to load the MAC register;              : out std_logic;  -- Control signal to load Y register
    O_loadOutA            : out std_logic;
    O_loadOutBaseline     : out std_logic;
    O_loadOutLissage       : out std_logic;
    O_SelFiltre           : out std_logic_vector(1 downto 0 );
    O_FilteredSampleValid : out std_logic);

end entity controlUnit;
architecture archi_operativeUnit of controlUnit is


  type T_state is (WAIT_SAMPLE, STORE, FILTRE_BASELINE, FILTRE_IRR_B, FILTRE_IRR_A, FILTRE_LISSAGE, OUTPUT, WAIT_END_SAMPLE);  -- state list
  signal SR_presentState : T_state;
  signal SR_futurState   : T_state;

begin


  process (I_clock, I_reset) is
    begin
      if I_reset = '1' then               -- asynchronous reset (active high)
        SR_presentState <= WAIT_SAMPLE;
      elsif rising_edge(I_clock) then     -- rising clock edge
        SR_presentState <= SR_futurState;
      end if;
    end process;
  
 
  -- Next state logic process
  process (SR_presentState, I_inputSampleValid, I_processingDoneIRRA, I_processingDoneIRRB, I_processingDoneBaseline, I_processingDoneLissage) is
    begin
      case SR_presentState is
  
        when WAIT_SAMPLE =>
          if I_inputSampleValid = '1' then
            SR_futurState <= STORE;
          else
            SR_futurState <= WAIT_SAMPLE;
          end if;
  
        when STORE =>
          O_initSum <= '1';
          O_initAddress <='1';
          SR_futurState <= FILTRE_BASELINE;
          O_SelFiltre <="00";
  
        when FILTRE_BASELINE =>
          if I_processingDoneBaseline= '1' then
            SR_futurState <= FILTRE_IRR_B;
            O_initSum <= '1';
            O_initAddress <='1';
            O_SelFiltre <="01";
          else
            SR_futurState <= FILTRE_BASELINE;
          end if;
  
          when FILTRE_IRR_B =>
          if I_processingDoneIRRB= '1' then
            SR_futurState <= FILTRE_IRR_A;
            O_initAddress <='1';
            O_SelFiltre <="10";
          else
            SR_futurState <= FILTRE_IRR_B;
          end if;

          when FILTRE_IRR_A =>
          if I_processingDoneIRRA= '1' then
            SR_futurState <= FILTRE_LISSAGE;
            O_initAddress <='1';
            O_initSum <= '1';
            O_SelFiltre<="11";
          else
            SR_futurState <= FILTRE_IRR_A;
          end if;

          when FILTRE_LISSAGE =>
          if I_processingDoneLissage= '1' then
            SR_futurState <= OUTPUT;

          else
            SR_futurState <= FILTRE_IRR_A;
          end if;

        when OUTPUT =>
          SR_futurState <= WAIT_END_SAMPLE;
  
        when WAIT_END_SAMPLE =>
          if I_inputSampleValid = '0' then
            SR_futurState <= WAIT_SAMPLE;
          else
            SR_futurState <= WAIT_END_SAMPLE;
          end if;
  
        when others =>
          SR_futurState <= WAIT_SAMPLE;
      end case;
    end process;
  
  -- Output logic
  O_loadShift           <= '1' when SR_presentState = STORE else '0';
  O_incrAddress         <= '1' when SR_presentState = FILTRE_BASELINE or SR_presentState = FILTRE_IRR_A or SR_presentState = FILTRE_IRR_B or SR_presentState = FILTRE_LISSAGE else '0';
  O_loadSum             <= '1' when SR_presentState = FILTRE_BASELINE or SR_presentState = FILTRE_IRR_A else '0';
  O_loadOutA            <= '1' when SR_presentState = FILTRE_IRR_A else '0';
  O_loadOutBaseline     <= '1' when SR_presentState = FILTRE_Baseline else '0';
  O_loadOutLissage      <= '1' when SR_presentState = OUTPUT else '0';
  O_FilteredSampleValid <= '1' when SR_presentState = OUTPUT else '0';

  --O_FilteredSampleValid <= _BLANK_;




end architecture archi_operativeUnit;