-------------------------------------------------------------------------------
-- Title      : operativeUnit
-- Project    :
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    :
-- Created    : 2018-04-11
-- Last update: 2019-02-13
-- Platform   :
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Operative unit of a sequential FIR filter. Including shift
-- register for samples, registers for coefficients, a MAC and a register to
-- store the result
-------------------------------------------------------------------------------
-- Copyright (c) 2018
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2019-02-13  1.1      marzel  Update to provide a 16-tap filter and improve
--                              the user experience ;)
-- 2018-04-11  1.0      jnbazin Created
-- 2018-04-18  1.0      marzel  Modification of SR_Y assignment to a round
--                              instead of a trunc
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity operativeUnit is

  port (
    I_clock          : in  std_logic;   -- global clock
    I_reset          : in  std_logic;   -- asynchronous global reset
    I_inputSample    : in  std_logic_vector(7 downto 0);  -- 8 bit input sample
    I_loadShift      : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_initAddress    : in  std_logic;  -- Control signal to initialize register read address
    I_incrAddress    : in  std_logic;  -- Control signal to increment register read address
    I_initSum        : in  std_logic;  -- Control signal to initialize the MAC register
    I_loadSum        : in  std_logic;  -- Control signal to load the MAC register;
    I_loadOutA        : in  std_logic;  
    I_loadOutBaseLine : in  std_logic;   
    I_loadOutLissage  : in  std_logic;
    I_SelFiltre       : in std_vector (1 downto 0)  ;
    O_processingDoneA : out std_logic;  
    O_processingDoneB : out std_logic;   
    O_processingDoneBaseLine: out std_logic;  
    O_processingDoneLissage : out std_logic;   
    O_total             : out std_logic_vector(11 downto 0)   
    );

end entity operativeUnit;

architecture arch_operativeUnit of operativeUnit is
  type registerFile is array(0 to 15) of signed(7 downto 0);
  signal SR_coefRegister : registerFile;


  signal SR_shiftRegister : registerFile;  -- shift register file used to store and shift input samples
  signal SC_multOperand1  : signed(7 downto 0);
  signal SC_multOperand2  : signed(7 downto 0);
  signal SC_MultResult    : signed(15 downto 0);  -- Result of the multiplication Xi*Hi
  signal SC_addResult     : signed(19 downto 0);  -- result of the accumulation addition
  signal SR_sum           : signed(19 downto 0);  -- Accumulation register
  signal SR_Y             : signed(7 downto 0);  -- filtered sample storage register
  signal SR_readAddress   : integer range 0 to 15;  -- register files read address



begin

-- Low-pass filter provided with octave (or Matlab ;)) command
--fir1(15, .001)/sqrt(sum(fir1(15, .001).^2))*2^6

SR_coefBaseLine <= (to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-2,12),        
                      to_signed(-2,12),        
                      to_signed(-2,12),        
                      to_signed(-3,12),        
                      to_signed(-3,12),        
                      to_signed(-3,12),        
                      to_signed(-4,12),        
                      to_signed(-4,12),        
                      to_signed(-5,12),        
                      to_signed(-5,12),        
                      to_signed(-6,12),        
                      to_signed(-6,12),        
                      to_signed(-7,12),        
                      to_signed(-7,12),        
                      to_signed(-8,12),        
                      to_signed(-8,12),        
                      to_signed(-9,12),        
                      to_signed(-10,12),       
                      to_signed(-10,12),       
                      to_signed(-11,12),       
                      to_signed(-11,12),       
                      to_signed(-12,12),       
                      to_signed(-13,12),       
                      to_signed(-13,12),       
                      to_signed(-14,12),       
                      to_signed(-14,12),       
                      to_signed(-15,12),       
                      to_signed(-15,12),       
                      to_signed(-16,12),       
                      to_signed(-16,12),       
                      to_signed(-17,12),       
                      to_signed(-17,12),       
                      to_signed(-18,12),       
                      to_signed(-18,12),       
                      to_signed(-18,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(1004,12),      
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-19,12),       
                      to_signed(-18,12),       
                      to_signed(-18,12),       
                      to_signed(-18,12),       
                      to_signed(-17,12),       
                      to_signed(-17,12),       
                      to_signed(-16,12),       
                      to_signed(-16,12),       
                      to_signed(-15,12),       
                      to_signed(-15,12),       
                      to_signed(-14,12),       
                      to_signed(-14,12),       
                      to_signed(-13,12),       
                      to_signed(-13,12),       
                      to_signed(-12,12),       
                      to_signed(-11,12),       
                      to_signed(-11,12),       
                      to_signed(-10,12),       
                      to_signed(-10,12),       
                      to_signed(-9,12),        
                      to_signed(-8,12),        
                      to_signed(-8,12),        
                      to_signed(-7,12),        
                      to_signed(-7,12),        
                      to_signed(-6,12),        
                      to_signed(-6,12),        
                      to_signed(-5,12),        
                      to_signed(-5,12),        
                      to_signed(-4,12),        
                      to_signed(-4,12),        
                      to_signed(-3,12),        
                      to_signed(-3,12),        
                      to_signed(-3,12),        
                      to_signed(-2,12),        
                      to_signed(-2,12),        
                      to_signed(-2,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12),        
                      to_signed(-1,12)                      
                      );

SR_coefA <= (to_signed(1554, 12), 
                      to_signed(-897, 12)
                      );

SR_coefB <= (to_signed(961, 12), 
                      to_signed(-1554, 12),
                      to_signed(961, 12)
                      );
                      

                      
 SR_coefLissage <=  (to_signed(-119,12),
                      to_signed(122,12),
                      to_signed(149,12),
                      to_signed(191,12),
                      to_signed(226,12),
                      to_signed(239,12),
                      to_signed(226,12),
                      to_signed(191,12),
                      to_signed(149,12),
                      to_signed(122,12),
                      to_signed(-119,12)
                      );

  shift : process (I_reset, I_clock) is
  begin  -- process shift
    if I_reset = '1' then               -- asynchronous reset (active high)
      SR_shiftRegister <= (others => (others => '0'));
    elsif Rising_edge(I_clock) then
      if I_loadShift = '1' then
--        for i in 1 to 15 loop
--          SR_shiftRegister(i) <= SR_shiftRegister(i-1);
--        end loop;
--        SR_shiftRegister(0) <= signed(I_inputSample);
        SR_shiftRegister(0) <= signed(I_inputSample);
        SR_shiftRegister(1 to 15) <= SR_shiftRegister(0 to 14);
      end if;
    end if;
  end process shift ;

  

  incr_address : process (I_reset, I_clock) is
  begin
    if I_reset = '1' then               -- asynchronous reset (active high)
      SR_readAddress <= 0;
    elsif Rising_edge(I_clock) then
      if I_initAddress = '1' then
        SR_readAddress <= 0;
      elsif I_incrAddress = '1' and SR_readAddress <= 15 then
        SR_readAddress <= SR_readAddress + 1;
      end if;
    end if;
  end process incr_address;

  O_processingDone <= '1' when SR_readAddress = 14 else '0';

  SC_multOperand1 <= SR_shiftRegister(SR_readAddress);   -- 8 bits
  SC_multOperand2 <= SR_coefRegister(SR_readAddress);    -- 8 bits
  SC_MultResult   <= SC_multOperand1 * SC_multOperand2;  -- 16 bits
  SC_addResult    <= resize(SC_MultResult, SC_addResult'length) + SR_sum;

  sum_acc : process (I_reset, I_clock) is
  begin
    if I_reset = '1' then               -- asynchronous reset (active high)
      SR_sum <= (others => '0');
    elsif Rising_edge(I_clock) then
      if I_initSum = '1' then
        SR_sum <= (others => '0');
      elsif I_loadSum = '1' then
        SR_sum <= SC_addResult;
      end if;
    end if;
  end process sum_acc;

  store_result : process (I_clock) is
  begin
    if Rising_edge(I_clock) then
      if I_loadY = '1' then
        if SC_addResult(6) = '1' then
            SR_Y <= SC_addResult(14 downto 7) + 1;
        else 
            SR_Y <= SC_addResult(14 downto 7);
      end if;
      end if;
    end if;

  end process store_result;

  O_Y <= std_logic_vector(SR_Y);

end architecture arch_operativeUnit;